#include <windows.h>
 
#define BUFFER_SIZE 256

//******************************************************************************
/// Configures COM port for specified baudrate.
/// @param[in]    port         COM port.
/// @param[in]    baudRate     Baudrate for communication.
/// @retval       TRUE         Success
///               FALSE        Failure
BOOL PortOpen(HANDLE *port, DWORD baudRate)
{
    DCB portDCB;                                              ///< COM port configuration structure
    BOOL returnValue = FALSE;
    COMMTIMEOUTS comTimeOut;
    /// Opens interface to reader.
    *port = CreateFile(TEXT("COM2:"), GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_FLAG_WRITE_THROUGH, NULL);
    if (*port == INVALID_HANDLE_VALUE)
    {
        printf("Error Opening COM Port.");
        return FALSE;
    }
 
    /// COM Port Configuration.
    portDCB.DCBlength = sizeof (DCB);                         ///< Initializes the DCBlength member.
    GetCommState (*port, &portDCB);                           ///< Gets the default port setting information.
    /// Changes the DCB structure settings.
    portDCB.BaudRate = baudRate;                              ///< Current baud. 
    portDCB.fBinary = TRUE;                                   ///< Binary mode; no EOF check. 
    portDCB.fParity = FALSE;                                  ///< Disable parity checking. 
    portDCB.fOutxCtsFlow = FALSE;                             ///< No CTS output flow control. 
    portDCB.fOutxDsrFlow = FALSE;                             ///< No DSR output flow control. 
    portDCB.fDtrControl = DTR_CONTROL_DISABLE;                ///< Disable DTR flow control type. 
    portDCB.fDsrSensitivity = FALSE;                          ///< DSR sensitivity. 
    portDCB.fTXContinueOnXoff = TRUE;                         ///< XOFF continues Tx. 
    portDCB.fOutX = FALSE;                                    ///< No XON/XOFF out flow control. 
    portDCB.fInX = FALSE;                                     ///< No XON/XOFF in flow control. 
    portDCB.fErrorChar = FALSE;                               ///< Disable error replacement. 
    portDCB.fNull = FALSE;                                    ///< Disable null stripping. 
    portDCB.fRtsControl = RTS_CONTROL_DISABLE;                ///< Disable RTS flow control. 
    portDCB.fAbortOnError = FALSE;                            ///< Do not abort reads/writes on error.
    portDCB.ByteSize = 8;                                     ///< Number of bits/byte, 4-8 
    portDCB.Parity = NOPARITY;                                ///< 0-4 = no, odd, even, mark, space. 
    portDCB.StopBits = ONESTOPBIT;                            ///< 0, 1, 2 = 1, 1.5, 2 
 
    /// Configures the port according to the specifications of the DCB structure.
    if (!SetCommState (*port, &portDCB))
    {
      printf("Error Configuring COM Port.");                 ///< Could not configure the serial port.      
      return FALSE;
    }
 
    /// Gets communication time out values.
    returnValue = GetCommTimeouts(*port, &comTimeOut);
    comTimeOut.ReadIntervalTimeout = 10;
    comTimeOut.ReadTotalTimeoutMultiplier = 1;
    comTimeOut.ReadTotalTimeoutConstant = 1;
    /// Set communication time out values
    returnValue = SetCommTimeouts(*port, &comTimeOut);

	/// Specifies the events to be monitored for a communication device.
	returnValue = SetCommMask(*port, EV_RXCHAR);
 
    return TRUE;
}
 
//******************************************************************************
/// Closes UART port.
/// @param[in]    port     COM port
/// @retval       TRUE     Success
///               FALSE    Failure
BOOL PortClose(HANDLE *port)
{
    if (*port == NULL)
    {
        return FALSE;
    }
    CloseHandle(*port);
    *port = NULL;
    return TRUE;
}

DWORD ReceiverThread(HANDLE portHandle)
{
	DWORD eventMask = 0;
	DWORD noOfBytesRead = 0;
	char receiveBuffer[BUFFER_SIZE] = {0};
	BOOL retVal = FALSE;

	while (TRUE)
	{
		memset(receiveBuffer, 0, BUFFER_SIZE);
		/// Waits for an event to occur for the port.
		retVal = WaitCommEvent(portHandle, &eventMask, NULL);
		if ((eventMask & EV_RXCHAR) == EV_RXCHAR)
		{
			retVal = ReadFile(portHandle, receiveBuffer, BUFFER_SIZE, &noOfBytesRead, NULL);
			if (retVal == TRUE)
			{
				if (noOfBytesRead != 0)
				{
					printf_s("\nData received (%u bytes): %s", noOfBytesRead, receiveBuffer);
				}
			}
			else
			{
				printf_s("\nComm port Read failure.");
				break;
			}
		}
		else
		{
			break;
		}
	}
	return TRUE;
}
 
//******************************************************************************
int wmain(void)
{
	HANDLE portHandle = NULL;
	HANDLE receiverThread = NULL;
    DWORD bytesTransmitted = 0;   
    char input = 0;
	BOOL retVal = FALSE;
	char transmitBuffer[BUFFER_SIZE] = {0};

    retVal = PortOpen(&portHandle, 9600);
    if (!retVal)
    {
        printf("\nCould not open COM port.");
		getchar();
		return FALSE;
    }   
    else
    {
		printf("COM port opened successfully.");
		/// Clears receive buffer.
		PurgeComm(portHandle, PURGE_RXCLEAR);
		/// Creates thread to monitor receive event.
		receiverThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ReceiverThread, portHandle, 0, NULL);
        while (TRUE)
        {
            printf("\n\nEnter 'y' to write on COM port or 'n' to exit.\t");
			fflush(stdin);
            scanf_s("%c", &input);

			if (input == 'y' || input == 'Y')
			{
				memset(transmitBuffer, 0, BUFFER_SIZE);
				printf("Enter data to send (Maximum %u characters).  ", BUFFER_SIZE);
				fflush(stdin);
				gets_s(transmitBuffer, BUFFER_SIZE);
				printf("Sending...");
				/// Writes data to COM port.
				retVal = WriteFile(portHandle, transmitBuffer, strlen(transmitBuffer), &bytesTransmitted, NULL);
				if (retVal == TRUE)
				{
					printf("\nData sent.");
				}
				else
				{
					printf("\nFailure, Error Code = %u", GetLastError());
				}
			}
			else
			{
				PortClose(&portHandle);
				Sleep(20);
                break;
			}           
        }
    }
}

